




                      MUSIC LIBRARY SYSTEM SAMPLE SOURCE CODE


              Some of the source code from the Music Library program is
         given below.  As you can see, the code is extensively commented,
         and should be relatively easy to understand even if you have never
         programmed before.

              The complete source code for all the programs in the Music
         Library System is available as discussed in the Introduction to the
         User Manual.  All of the source code is just as comprehensively
         annotated as the code you will find below.  In fact, the comments
         are so extensive that we have created a separate file for the docu-
         mented source code.  The working code that you will use to write
         your programs is minimally commented so your screen won't be filled
         with comments when you are working with it.  You can then print the
         annotated source code which has one procedure per page, file the
         printed routines alphabetically in a notebook, and thus be able to
         quickly find the commented source code for any part of the program
         you are working on.














































              The "Add" routine is used to add records to the file using
         the "A" option from the "Update" menu.  This routine shows how
         the index files, some of which are a combination of three keys,
         are built.


***************************************************************************
***************************************************************************


         (*  Add - Adds a record to the file.                       *)
         (*                                                         *)
         (*  The caller passes two parameters to this procedure.    *)
         (*  The first is the record buffer that will be used by    *)
         (*  the "InputCust" routine, "AddRec", etc.; the           *)
         (*  second is the "RecNumber" in which the "Add" routine   *)
         (*  will return the number of the record in the file       *)
         (*  that was used for the added record.  Both of the       *)
         (*  parameters are prefaced by the "var" statement so      *)
         (*  any new data placed into these areas by this routine   *)
         (*  will be returned to the caller.                        *)


procedure Add(var MastBuffer : MastRec; var RecNumber : Integer);

const
  Spaces =  '                         ';
begin
  ComposerTitleKey := '';     (* Clear the work areas used to build the   *)
  Composer1Key := '';         (* keys for the Index files.  These are     *)
  Composer2Key := '';         (* global areas defined in the Main module. *)
  with MastBuffer do          (* Define the buffer used within the "begin *)
  begin                       (* ... end" block. Then call "ClearFrame"   *)
    ClearFrame;               (* procedure to clear lines 4 - 21 on the   *)
    Outform;                  (* screen.  Call "OutForm" to display the   *)
                              (* "Update" data entry screen for the Music *)
                              (* Master Record.                           *)

        (* "FillChar" is a Pascal procedure that will fill the first      *)
        (* parameter with the value defined in the third parameter for    *)
        (* the number of bytes specified in the second parameter.  The    *)
        (* "SizeOf" statement is a Pascal procedure that returns the      *)
        (* number of bytes occupied in memory by the variable passed to   *)
        (* the function.  See the "Turbo Reference Manual" for these      *)
        (* commands.  Thus, the "MastBuffer" area is filled with zeros    *)
        (* for the size of the buffer area.                               *)

    FillChar(MastBuffer,SizeOf(MastBuffer),0);
















        (*  Call "InputCust" procedure to read data entered on "Update"   *)
        (*  screen and move it to fields in the "MastBuffer" area.  Then  *)
        (*  use the Database Toolbox "AddRec" procedure to add a record  *)
        (*  to the file using the data in the buffer.  The "AddRec"       *)
        (*  procedure returns the number of the added record in the       *)
        (*  "RecNumber" parameter. See the "Database Toolbox" manual for  *)
        (*  the "AddRec" procedure.  We will use the "RecNumber" to build *)
        (*  the Index files.  Close the Music Masterfile immediately      *)
        (*  after writing the record to save the data, then reopen it.    *)

    InputCust(MastBuffer);
    AddRec(DatF,RecNumber,MastBuffer);
    CloseFile(DatF);
    OpenFile(DatF,'MUSIC.DAT',MastRecSize);



        (*  Now build the six Index files using the "RecNumber" to store  *)
        (*  the location of the Master Record in the Index files.  Use    *)
        (*  the Pascal "+" function to concatenate fields into one string *)
        (*  for the combined Index file keys.  "Copy" is a Pascal string  *)
        (*  function that returns a substring containing the number of    *)
        (*  characters in the third parameter from the string specified   *)
        (*  in the first parameter, starting at the position given by the *)
        (*  second parameter.  "Length" is a Pascal string function that  *)
        (*  returns the number of characters in the string specified.     *)
        (*  See the "Turbo Reference Manual" for these commands.  Thus    *)
        (*  the "ComposerTitleKey" is built by moving the "Composer"      *)
        (*  field from the buffer into the key area; then the "Copy"      *)
        (*  function will move spaces into the area immediately after the *)
        (*  "Composer" starting from position 1 in the "spaces" area for  *)
        (*  a length of 20 bytes, minus the length of the data in the     *)
        (*  "Composer" field.  If the data in the "Composer" field is 10  *)
        (*  bytes, then 10 bytes of spaces will be moved into the         *)
        (*  "ComposerTitleKey" area beginning immediately after the       *)
        (*  data from the "Composer" field. Then the first 40 bytes from  *)
        (*  the "Title" field in the record are added to the key work     *)
        (*  area to complete the 60-byte key. Then use the Database       *)
        (*  Toolbox "AddKey" procedure to build a record in each Index    *)
        (*  file containing the "RecNumber" of the Master Record and the  *)
        (*  key.  See the "Database Toolbox" manual for this command.     *)
        (*  Single keys can be added simply by passing the procedure      *)
        (*  the key from the buffer: i.e., "CategoryCode," "Title," etc.  *)

    ComposerTitleKey := Composer + Copy(Spaces,1,20 - Length(Composer))
                        + Copy(Title,1,40);
    AddKey(ComposerTitleIndexFile,RecNumber,ComposerTitleKey);
    AddKey(CategoryIndexFile,RecNumber,CategoryCode);
    AddKey(TitleIndexFile,RecNumber,Title);
    AddKey(CatalogueIndexFile,RecNumber,CatalogueNbr);
    Composer1Key := Composer + Copy(Spaces,1,20 - Length(Composer))
                    + CategoryCode + Copy(Spaces,1,3 -
                    Length(CategoryCode)) + Copy(Title,1,37);
    AddKey(Composer1IndexFile,RecNumber,Composer1Key);
    Composer2Key := Composer + Copy(Spaces,1,20 - Length(Composer))
                    + CategoryCode + Copy(Spaces,1,3 -
                    Length(CategoryCode)) + Copy(CatalogueNbr,1,17);
    AddKey(Composer2IndexFile,RecNumber,Composer2Key);










        (*  Set the terminal to low video, move the cursor to position    *)
        (*  61 on line 2, and write the new number of records in the file *)
        (*  to the cursor position on the screen using the Database       *)
        (*  Toolbox "UsedRecs" function which returns the number of       *)
        (*  active records in the file.  The ":5" in the "Write" command  *)
        (*  forces the program to write 5 characters to the screen.       *)
        (*  Reset the terminal to normal video, and we're done.           *)

    LowVideo;
    GotoXY(61,2); Write(UsedRecs(DatF) :5);
    NormVideo;
  end;     (* end of "With MastBuffer do"     *)
end;       (* end of "Add" procedure          *)


***************************************************************************
***************************************************************************



              Modifying the code to produce your own database can be done
         without too much difficulty.  Let's say you want to create a data-
         base that will enable you to display all the manufacturing
         companies in a range of zip codes having gross sales of $10 to
         $20 million.

              This is similar to the "Composer, Category Code, and Title"
         "Display Range" command.  Since the "Display Range" commands use
         the index files to very quickly locate the requested range of
         records, the first thing you must do is build the index file
         record required to display the requested range of data.  That
         task is accomplished by the "Add" procedure.

              The index file record will have the Company (or SIC) Code
         (an arbitrary range of numbers representing manufacturing, banking,
         retailing, etc. businesses similar to the Music Category Codes) as
         the primary key, followed by the Zip Code, and the Sales Code as
         secondary keys.  Then the "AddKey" procedure will take care of
         sorting the records by Sales Code (a three-digit number repre-
         senting sales in millions) within each Zip Code, and by Zip Code
         within each Company Code.

              The Composer, Category, and Title index file built in the
         "Add" procedure has precisely that structure.  Thus you could
         simply change the reference to "Composer" in the code to
         "CompanyCode," change "CategoryCode" to "ZipCode," and change
         the "Title" to "SalesCode."



















              The Turbo Pascal text editor has a search and replace
         function that can take care of this automatically for you.  Just
         give the editor the old and the new text strings, and it will
         search the code, replacing every occurrence of "Composer" with
         "CompanyCode" and so on.  Then change the lengths in the "Copy"
         commands to conform to the lengths of the new data fields.  The
         code to build the "Composer1Key" would be changed to read:

             CompanyZipSalesKey := CompanyCode + Copy(Spaces,1,5 -
                                   Length(CompanyCode)) + ZipCode
                                   + Copy(Spaces,1,10 - Length(ZipCode))
                                   + Copy(SalesCode,1,3);

              Having built the index file, you would then simply modify
         the  "Composer, Category, and Title" Display Range command
         in the same manner.  You would build the key from the data
         input by the user in the same way as in the "Add" procedure,
         and then simply call the "SearchKey" procedure to find the first
         record in the index file having a key equal to or greater than
         the key you pass to it.  Then it's a trivial matter to check the
         SalesCode area in each key for the range of "10" to "20" and
         exclude any records not in that range.  For the index file
         records that fall within the desired range, you would simply
         call the "GetRec" procedure, passing the procedure the record
         number that was returned to you by the "SearchKey" procedure.
         Display the record by calling another procedure, get the next
         index file record by calling "NextKey" and continue the process
         until "NextKey" returns a key greater than the terminating key
         entered by the user.
